import common
import struct


def FullOTA_InstallEnd(info):
  try:
    bootloader_img = info.input_zip.read("RADIO/motoboot.bin")
  except KeyError:
    print "no motoboot.bin in target_files; skipping install"
  else:
    WriteBootloader(info, bootloader_img)


def IncrementalOTA_InstallEnd(info):
  try:
    target_bootloader_img = info.target_zip.read("RADIO/motoboot.bin")
    try:
      source_bootloader_img = info.source_zip.read("RADIO/motoboot.bin")
    except KeyError:
      source_bootloader_img = None

    if source_bootloader_img == target_bootloader_img:
      print "bootloader unchanged; skipping"
    else:
      WriteBootloader(info, target_bootloader_img)
  except KeyError:
    print "no motoboot.bin in target target_files; skipping install"

# /* motoboot.bin format */
#
# struct img_info_hdr
# {
#        char img_name[24];
#
#        uint32_t start_block;
#        uint32_t end_block;
# };
#
# struct file_hdr
# {
#        uint32_t num_elements;
#        struct img_info_hdr img_hdr[15];
#        uint8_t extra_buffer[540];
# };
#
# motoboot.bin contains 5 separate images:
#    mbr, mbm, ebr, cdt.bin, logo.bin
#

release_partitions = "mbm ebr cdt.bin logo.bin"
debug_partitions = "mbm ebr cdt.bin logo.bin"

block_size = 131072

def WriteBootloader(info, bootloader):
  info.script.Print("Writing bootloader...")

  header_fmt = "I"
  header_size = struct.calcsize(header_fmt)
  num_images, = struct.unpack(header_fmt, bootloader[:header_size])

  img_info_fmt = "<24sII"
  img_info_size = struct.calcsize(img_info_fmt)

  imgs = [struct.unpack(img_info_fmt,
                        bootloader[header_size+i*img_info_size:
                                     header_size+(i+1)*img_info_size])
          for i in range(num_images)]

  total = 0
  p = header_size + 15 * img_info_size + 540
  img_dict = {}
  for name, start, end in imgs:
    size = (end - start + 1) * block_size
    img_dict[trunc_to_null(name)] = p, size
    p += size
  assert p == len(bootloader), "motoboot.bin corrupted"
  imgs = img_dict

  common.ZipWriteStr(info.output_zip, "bootloader-flag.txt",
                     "updating-bootloader" + "\0" * 13)
  common.ZipWriteStr(info.output_zip, "bootloader-flag-clear.txt", "\0" * 32)

  _, misc_device = common.GetTypeAndDevice("/misc", info.info_dict)

  info.script.AppendExtra(
      'package_extract_file("bootloader-flag.txt", "%s");' %
      (misc_device,))

  # Depending on the build fingerprint, we can decide which partitions
  # to update.
  fp = info.info_dict["build.prop"]["ro.build.fingerprint"]
  if "release-keys" in fp:
    to_flash = release_partitions.split()
  else:
    to_flash = debug_partitions.split()

  # Write the images to separate files in the OTA package
  for i in to_flash:
    try:
      _, device = common.GetTypeAndDevice("/"+i, info.info_dict)
    except KeyError:
      print "skipping flash of %s; not in recovery.fstab" % (i,)
      continue
    common.ZipWriteStr(info.output_zip, "bootloader.%s.img" % (i,),
                       bootloader[imgs[i][0]:imgs[i][0]+imgs[i][1]])

    info.script.AppendExtra('package_extract_file("bootloader.%s.img", "%s");' %
                            (i, device))

  info.script.AppendExtra(
      'package_extract_file("bootloader-flag-clear.txt", "%s");' %
      (misc_device,))

def trunc_to_null(s):
  if '\0' in s:
    return s[:s.index('\0')]
  else:
    return s
